import compose from './compose';

var METADATA = Symbol('recursive-func@METADATA');

var applyMiddlewares = function applyMiddlewares(middlewares, func) {
  return middlewares.length !== 0 ? compose(middlewares)(func) : func;
};

var unwrapMetadata = function unwrapMetadata(func) {
  return func[METADATA];
};

export var recursive = function recursive(func) {
  var _Object$assign;

  var middlewares = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var wrapper = applyMiddlewares(middlewares, func(function () {
    return wrapper.apply(undefined, arguments);
  }));

  return Object.assign(wrapper, (_Object$assign = {}, _Object$assign[METADATA] = {
    func: func,
    middlewares: middlewares
  }, _Object$assign));
};

export var withMiddlewares = function withMiddlewares(middlewares, wrapper) {
  var metadata = unwrapMetadata(wrapper);

  return recursive(metadata.func, [].concat(metadata.middlewares, middlewares));
};